#ifndef ASTAR_NODE_H
#define ASTAR_NODE_H

/// \author Randy Ribler
/// \date Oct, 2004

// Disable warnings about stl names being too long.
#pragma warning(disable:4786)

#include <string>
using namespace std;

class AStar;

class AStarNode
{
   public:
      // Default Constructor
      AStarNode();

      // Constructor gets cost (depth) as a parameter.
      AStarNode(float gx_CostToHere);

      // Pure virtual functions.  These must all be overriden in the
      // subclass that solves a particular problem using AStar.

      // This is where most of the work will be when you create a new AStar
      // application.  GenerateChildren needs to do the following:
      //  for each child
      //     create a node for the child and store its state
      //     set the parent node to point to the parent.
      //     set hx_estimateToGoal_
      //     set gx_costToHere_
      //     set fx_costFromRootToGoal_
      //     use aStar->addToOpen(newNode) to add the child to the open
      //     list
      //  return the number of children generated.
      virtual int generateChildren(AStar* aStar) const = 0;

      // Determine if this node has the same state as another node.
      virtual bool sameState(const AStarNode& node) const = 0;

      // Create a copy of this node (using new) and return a pointer to it.
      // AStarNode* NPuzzleNode::clone() const
      // {
      //    return new NPuzzleNode(*this);
      // }
      virtual AStarNode* clone() const = 0;

      // Create a string that UNIQUELY identifies this state.
      // 1 2 3
      // 4 5 6
      // 7 8 0
      // "123456780"
      virtual string serializeState() const = 0;

      // Operations for comparing two nodes.

      // To be equal the nodes must have the same state.
      virtual bool operator==(const AStarNode& rhs) const;
      virtual bool operator!=(const AStarNode& rhs) const;

      // These function compare the relative values of
      // fx_costFromRootToGoal_
      virtual bool operator<=(const AStarNode& rhs) const;
      virtual bool operator>=(const AStarNode& rhs) const;
      virtual bool operator<(const AStarNode& rhs) const;
      virtual bool operator>(const AStarNode& rhs) const;

      // Destructor
      virtual ~AStarNode();

      // Return a pointer to the node's parent.
      const AStarNode* getParent() const;

      // Set the parent for the node
      void setParent(const AStarNode* parent);

      // Return value for fx_costFromRootToGoal_
      float getFxCostFromRootToGoal() const;

   protected:

      // Estimated cost of path through this node from root to goal.
      float fx_costFromRootToGoal_;

      // Cost of getting to this node.
      float gx_costToHere_;

      // Estimated cost from here to the goal.
      float hx_estimateToGoal_;

      // Pointer to the parent node. (null for the root node.)
      const AStarNode* parent_;
};

#endif
